package net.christophe.genin.spring.boot.pdfgenerator.core;


import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = TestApplication.class)
public class PdfBuilderTest {

    public static final String TARGET_TMP_TEST = "target/tmp-test";

    @Autowired
    PdfGeneratorBuilder builder;
    private Map<String, Object> parameters;

    private SectionView section1;
    private SectionView section2;

    public class Pair {
        public Pair(String key, String value) {
            this.key = key;
            this.value = value;
        }
        public String key;
        public String value;
    }

    public class SectionView {
        public boolean isEmpty = false;

        public String seq = "0.0";

        public String title = "No title";

        /*
         * 1. 先处理xhtml， replace {{1.1_1}} with {{1.1_score}}
         * 2. 添加 {{1.1_score}} 等到 parameters
         * 3. xhtml  key: value.
         * 
         * 表格变量顺序从左到右，从上到下。合并表格取左上。
         */
        public List<Pair> data = new ArrayList<>();

        public List<SectionView> views = new ArrayList<>();

        public Map<String, String> getKeyMapping() {
            HashMap<String, String> ret = new HashMap<>();
            for(int i = 0; i < data.size(); i++) {
                ret.put(
                    String.format("%s_%d", this.seq, i), 
                    String.format("%s_%s", this.seq, data.get(i).key));
            }
            for(SectionView sv: this.views) {
                ret.putAll(sv.getKeyMapping());
            }
            return ret;
        }

        public Map<String, Object> getKeyValues() {
            HashMap<String, Object> ret = new HashMap<>();
            ret.put(String.format("%s__seq", this.seq), this.seq);
            ret.put(String.format("%s__title", this.seq), this.title);
            for(int i = 0; i < data.size(); i++) {
                ret.put(
                    String.format("%s_%s", this.seq, data.get(i).key),
                    data.get(i).value
                );
            }
            for(SectionView sv: this.views) {
                ret.putAll(sv.getKeyValues());
            }
            return ret;
        }
    }

    public class SectionTable extends SectionView {
        @Override
        public Map<String, String> getKeyMapping() {
            HashMap<String, String> ret = new HashMap<>();
            for(SectionView sv: this.views) {
                if(sv instanceof SectionTableRow) {
                    for(Map.Entry<String, String> entry: sv.getKeyMapping().entrySet()) {
                        ret.put(this.seq + "_" + entry.getKey(), this.seq + "_" + entry.getValue());
                    }
                    break;  // all rows are the same.
                } else {
                    throw new RuntimeException("SectionTable should only contain SectionTableRow");
                }
            }
            return ret;
        }

        @Override
        public Map<String, Object> getKeyValues() {
            Map<String, Object> ret = new HashMap<>();
            for(int i = 0; i < this.views.size(); i++) {
                SectionView sv = this.views.get(i);
                HashMap<String, Object> mp = new HashMap<>();
                if(sv instanceof SectionTableRow) {
                    for(Map.Entry<String, String> entry: sv.getKeyMapping().entrySet()) {
                        mp.put(this.seq + "_" + entry.getKey(), entry.getValue());
                    }
                } else {
                    throw new RuntimeException("SectionTable should only contain SectionTableRow");
                }
                ret.put(Integer.toString(i), mp);
            }
            return ret;
        }
    }

    public class SectionTableRow extends SectionView {
        @Override
        public Map<String, String> getKeyMapping() {
            HashMap<String, String> ret = new HashMap<>();
            for(int i = 0; i < data.size(); i++) {
                ret.put(
                    String.format("%d", i), 
                    String.format("%s", data.get(i).key));
            }
            return ret;
        }

        @Override
        public Map<String, Object> getKeyValues() {
            HashMap<String, Object> ret = new HashMap<>();
            for(int i = 0; i < data.size(); i++) {
                ret.put(data.get(i).key, data.get(i).value);
            }
            return ret;
        }
    }

    @Before
    public void initialize() {
        new File(TARGET_TMP_TEST).mkdirs();

        parameters = new HashMap<>();
        parameters.put("next", "42");

        section1 = new SectionView();
        section1.seq = "1";
        section1.title = "报告概述";

        section1.views.add(new SectionView());
        SectionView section1_1 = section1.views.get(0);
        section1_1.seq = "1.1";
        section1_1.title = "审核结果";
        section1_1.data.add(new Pair("score", "77"));
        section1_1.data.add(new Pair("result", "已通过"));
        section1_1.data.add(new Pair("name", "上海成泰医药科技有限公司"));
        section1_1.data.add(new Pair("date", "2025年2月 26日"));
        section1_1.data.add(new Pair("version", "PLUS版"));

        section1.views.add(new SectionView());
        SectionView section1_2 = section1.views.get(1);
        section1_2.seq = "1.2";
        section1_2.title = "风险概述";
        section1_2.data.add(new Pair("tips", "-"));
        section1_2.data.add(new Pair("suggestion", "审核试用"));

        section1.views.add(new SectionView());
        SectionView section1_3 = section1.views.get(2);
        section1_3.seq = "1.3";
        section1_3.title = "评分情况";
        section1_3.data.add(new Pair("typeScore", "3"));
        section1_3.data.add(new Pair("ageScore", "3"));
        section1_3.data.add(new Pair("claimCapitalScore", "3"));
        section1_3.data.add(new Pair("realCapitalScore", "3"));
        section1_3.data.add(new Pair("publicRecordScore", "20"));
        section1_3.data.add(new Pair("financeScore", "40"));
        section1_3.data.add(new Pair("antiBribeScore", "5"));

        section2 = new SectionView();
        section2.seq = "2";
        section2.title = "注册信息";

        section2.views.add(new SectionView());
        SectionView section2_1 = section2.views.get(0);
        section2_1.seq = "2.1";
        section2_1.title = "注册信息";
        section2_1.data.add(new Pair("riskCount", "0"));
        section2_1.data.add(new Pair("companyName", "上海成泰医药科技有限公司"));
        section2_1.data.add(new Pair("industry", "-"));
        section2_1.data.add(new Pair("juridicalPerson", "闫天牧"));
        section2_1.data.add(new Pair("claimCapitalScore", ""));
        section2_1.data.add(new Pair("realCapitalScore", ""));
        section2_1.data.add(new Pair("companySize", ""));
        section2_1.data.add(new Pair("employee", ""));
        section2_1.data.add(new Pair("status", ""));
        section2_1.data.add(new Pair("govAgency", ""));
        section2_1.data.add(new Pair("establishDate", ""));
        section2_1.data.add(new Pair("registeredAddress", ""));
        section2_1.data.add(new Pair("taxCert", ""));
        section2_1.data.add(new Pair("companyType", ""));
        section2_1.data.add(new Pair("scope", ""));
        section2_1.data.add(new Pair("companyTypeStd", "个体户、民营企业 0分\n" + //
                        "上市公司、国有企业、跨国企业 10分\n" + //
                        "除以上两种 5分"));
        section2_1.data.add(new Pair("companyTypeDesc", "有限责任公司（自然人投资或控股）"));
        section2_1.data.add(new Pair("companyTypeResult", "合格"));
        section2_1.data.add(new Pair("companyTypeScore", "3"));
        section2_1.data.add(new Pair("activePeriodStd", "3年及以下 0分\n" + //
                        "3-5年，含3年 5分\n" + //
                        "5年以上，含5年 10分"));
        section2_1.data.add(new Pair("activePeriodDesc", "3年"));
        section2_1.data.add(new Pair("activePeriodResult", "合格"));
        section2_1.data.add(new Pair("activePeriodScore", "3"));
        section2_1.data.add(new Pair("claimCapitalStd", "100万以下 0分\n" + //
                        "100万-500万 3分\n" + //
                        "500万以上 5分"));
        section2_1.data.add(new Pair("claimCapitalDesc", "300 万人民币"));
        section2_1.data.add(new Pair("claimCapitalResult", "合格"));
        section2_1.data.add(new Pair("claimCapitalScore", "3"));
        section2_1.data.add(new Pair("realCapitalStd", "100万以下 0分\n" + //
                        "100万-500万 3分\n" + //
                        "500万以上 5分"));
        section2_1.data.add(new Pair("realCapitalDesc", "102 万人民币"));
        section2_1.data.add(new Pair("realCapitalResult", "合格"));
        section2_1.data.add(new Pair("realCapitalScore", "3"));

        section2.views.add(new SectionView());
        SectionView section2_2 = section2.views.get(1);
        section2_2.seq = "2.2";
        section2_2.title = "利益冲突情况";
        section2_2.data.add(new Pair("riskCount", "0"));
    }

    private String preprocess(SectionView sv, String path) throws IOException {
        File file = new File(path.replace(".xhtml", ".processed.xhtml"));
        if (file.exists()) {
            file.delete();
        }
        BufferedReader reader = new BufferedReader(new FileReader(path));
        String fileContent = reader.lines().collect(Collectors.joining(System.lineSeparator()));
        reader.close();
        for(Map.Entry<String, String> entry : sv.getKeyMapping().entrySet()) {
            fileContent = fileContent.replace(entry.getKey(), entry.getValue());
        }
        BufferedWriter writer = new BufferedWriter(new FileWriter(file.toString()));
        writer.write(fileContent);
        writer.close();
        return file.toString();
    }

    @Test
    public void should_generate_pdf_from_file_system() {
        Generator generator = builder.create().withFileSystem("src/test/resources/test.html").build();
        assertNotNull(generator);
        byte[] bytes = generator.toBytes(parameters);
        assertNotNull(bytes);
        assertTrue(bytes.length > 0);
    }

    @Test
    public void should_generate_pdf_from_classpath() {
        Generator generator = builder.create()
                .withClasspathResource("test.html")
                .build();
        assertNotNull(generator);
        byte[] bytes = generator.toBytes(parameters);
        assertNotNull(bytes);
        assertTrue(bytes.length > 0);
    }

    @Test(expected = Exception.class)
    public void should_throw_exception_if_no_type() {
        builder.create().build();
    }

    @Test
    public void should_generate_pdf_from_string() {
        Generator generator = builder.create()
                .withCompilerOptions(template -> {
                    return template.escapeHTML(true);
                })
                .withXhtml("<html><body>The response is {{next}}</body></html>")
                .build();
        assertNotNull(generator);
        byte[] bytes = generator.toBytes(parameters);
        assertNotNull(bytes);
        assertTrue(bytes.length > 0);
    }

    @Test
    public void test_cso_report() throws IOException {
        String tmpl = "src/test/resources/cso_report.xhtml";
        String processedTmpl = preprocess(section1, tmpl);

        Generator generator = builder.create().withFileSystem(processedTmpl).build();
        File file = new File(TARGET_TMP_TEST + "/cso_report.pdf");
        if (file.exists()) {
            file.delete();
        }

        parameters.putAll(section1.getKeyValues());
        generator.toFile(parameters, file);
        assertTrue(file.exists());
    }
}
